@@ -31,49 +31,12 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <Integer.h>
 #include <std.h>
 #include <ctype.h>
-#include <float.h>
 #include <limits.h>
-#include <math.h>
 #include <Obstack.h>
 #include <AllocRing.h>
 #include <new.h>
 #include <builtin.h>
-
-#ifndef HUGE_VAL
-#ifdef HUGE
-#define HUGE_VAL HUGE
-#else
-#define HUGE_VAL DBL_MAX
-#endif
-#endif
-
-/*
- Sizes of shifts for multiple-precision arithmetic.
- These should not be changed unless Integer representation
- as unsigned shorts is changed in the implementation files.
-*/
-
-#define I_SHIFT         (sizeof(short) * CHAR_BIT)
-#define I_RADIX         ((unsigned long)(1L << I_SHIFT))
-#define I_MAXNUM        ((unsigned long)((I_RADIX - 1)))
-#define I_MINNUM        ((unsigned long)(I_RADIX >> 1))
-#define I_POSITIVE      1
-#define I_NEGATIVE      0
-
-/* All routines assume SHORT_PER_LONG > 1 */
-#define SHORT_PER_LONG  ((unsigned)(((sizeof(long) + sizeof(short) - 1) / sizeof(short))))
-#define CHAR_PER_LONG   ((unsigned)sizeof(long))
-
-/*
-  minimum and maximum sizes for an IntRep
-*/
-
-#define MINIntRep_SIZE   16
-#define MAXIntRep_SIZE   I_MAXNUM
-
-#ifndef MALLOC_MIN_OVERHEAD
-#define MALLOC_MIN_OVERHEAD 4
-#endif
+#include "Integer.hP"
 
 IntRep _ZeroRep = {1, 0, 1, {0}};
 IntRep _OneRep = {1, 0, 1, {1}};
@@ -413,102 +376,6 @@ int Iislong(const IntRep* rep)
     return 0;
 }
 
-// convert to a double 
-
-double Itodouble(const IntRep* rep)
-{ 
-  double d = 0.0;
-  double bound = DBL_MAX / 2.0;
-  for (int i = rep->len - 1; i >= 0; --i)
-  {
-    unsigned short a = I_RADIX >> 1;
-    while (a != 0)
-    {
-      if (d >= bound)
-        return (rep->sgn == I_NEGATIVE) ? -HUGE_VAL : HUGE_VAL;
-      d *= 2.0;
-      if (rep->s[i] & a)
-        d += 1.0;
-      a >>= 1;
-    }
-  }
-  if (rep->sgn == I_NEGATIVE)
-    return -d;
-  else
-    return d;
-}
-
-// see whether op double() will work-
-// have to actually try it in order to find out
-// since otherwise might trigger fp exception
-
-int Iisdouble(const IntRep* rep)
-{
-  double d = 0.0;
-  double bound = DBL_MAX / 2.0;
-  for (int i = rep->len - 1; i >= 0; --i)
-  {
-    unsigned short a = I_RADIX >> 1;
-    while (a != 0)
-    {
-      if (d > bound || (d == bound && (i > 0 || (rep->s[i] & a))))
-        return 0;
-      d *= 2.0;
-      if (rep->s[i] & a)
-        d += 1.0;
-      a >>= 1;
-    }
-  }
-  return 1;
-}
-
-// real division of num / den
-
-double ratio(const Integer& num, const Integer& den)
-{
-  Integer q, r;
-  divide(num, den, q, r);
-  double d1 = q.as_double();
- 
-  if (d1 >= DBL_MAX || d1 <= -DBL_MAX || sign(r) == 0)
-    return d1;
-  else      // use as much precision as available for fractional part
-  {
-    double  d2 = 0.0;
-    double  d3 = 0.0; 
-    int cont = 1;
-    for (int i = den.rep->len - 1; i >= 0 && cont; --i)
-    {
-      unsigned short a = I_RADIX >> 1;
-      while (a != 0)
-      {
-        if (d2 + 1.0 == d2) // out of precision when we get here
-        {
-          cont = 0;
-          break;
-        }
-
-        d2 *= 2.0;
-        if (den.rep->s[i] & a)
-          d2 += 1.0;
-
-        if (i < r.rep->len)
-        {
-          d3 *= 2.0;
-          if (r.rep->s[i] & a)
-            d3 += 1.0;
-        }
-
-        a >>= 1;
-      }
-    }
-
-    if (sign(r) < 0)
-      d3 = -d3;
-    return d1 + d3 / d2;
-  }
-}
-
 // comparison functions
   
 int compare(const IntRep* x, const IntRep* y)
