@@ -23,7 +23,7 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #endif
 #define _String_h 1
 
-#include <stream.h>
+#include <iostream.h>
 #include <Regex.h>
 
 struct StrRep                     // internal String representations
