@@ -26,22 +26,27 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
  *	Sam Kendall added C++.
  */
 
-#include <stdio.h>
-#include <ctype.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-
-#include "getopt.h"
-
+/* AIX requires this to be the first thing in the file. */
 #ifdef	__GNUC__
 #define	alloca	__builtin_alloca
 #else
 #ifdef	sparc
 #include <alloca.h>
 #else
+#ifdef _AIX
+ #pragma alloca
+#else
 extern char *alloca ();
 #endif
 #endif
+#endif
+
+#include <stdio.h>
+#include <ctype.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+
+#include "getopt.h"
 
 extern char *malloc (), *realloc ();
 extern char *getenv ();
