@@ -24,11 +24,13 @@ the executable file might be covered by the GNU General Public License. */
 
 /*  written by Per Bothner (bothner@cygnus.com) */
 
+#define _POSIX_SOURCE
 #include "libioP.h"
 #include <fcntl.h>
-#include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <string.h>
+#include <errno.h>
 #ifndef errno
 extern int errno;
 #endif
@@ -100,7 +102,7 @@ _IO_file_close_it(fp)
     return EOF;
 
   /* This flushes as well as switching mode. */
-  if (fp->_pptr > fp->_pbase || _IO_in_put_mode(fp))
+  if (fp->_IO_write_ptr > fp->_IO_write_base || _IO_in_put_mode(fp))
     if (_IO_switch_to_get_mode(fp))
 	  return EOF;
 
@@ -127,7 +129,6 @@ _IO_file_finish(fp)
 {
   if (!(fp->_flags & _IO_DELETE_DONT_CLOSE))
     _IO_file_close_it(fp);
-  _IO_un_link(fp);
   _IO_default_finish(fp);
 }
 
@@ -200,8 +201,9 @@ _IO_file_setbuf(fp, p, len)
     if (_IO_default_setbuf(fp, p, len) == EOF)
 	return EOF;
 
-    fp->_pbase = fp->_pptr = fp->_epptr = fp->_base;
-    _IO_setg(fp, fp->_base, fp->_base, fp->_base);
+    fp->_IO_write_base = fp->_IO_write_ptr = fp->_IO_write_end
+      = fp->_IO_buf_base;
+    _IO_setg(fp, fp->_IO_buf_base, fp->_IO_buf_base, fp->_IO_buf_base);
 
     return 0;
 }
@@ -222,9 +224,9 @@ _IO_do_write(fp, data, to_do)
        Instead, just indicate that offset (before and after) is
        unpredictable. */
     fp->_offset = _IO_pos_BAD;
-  else if (fp->_egptr != fp->_pbase)
+  else if (fp->_IO_read_end != fp->_IO_write_base)
     { 
-      _IO_pos_t new_pos = fp->_jumps->__seek(fp, fp->_pbase - fp->_egptr, 1);
+      _IO_pos_t new_pos = fp->_jumps->__seek(fp, fp->_IO_write_base - fp->_IO_read_end, 1);
       if (new_pos == _IO_pos_BAD)
 	return EOF;
       fp->_offset = new_pos;
@@ -232,9 +234,10 @@ _IO_do_write(fp, data, to_do)
   count = fp->_jumps->__write(fp, data, to_do);
   if (fp->_cur_column)
     fp->_cur_column = _IO_adjust_column(fp->_cur_column - 1, data, to_do) + 1;
-  _IO_setg(fp, fp->_base, fp->_base, fp->_base);
-  fp->_pbase = fp->_pptr = fp->_base;
-  fp->_epptr = (fp->_flags & _IO_LINE_BUF+_IO_UNBUFFERED) ? fp->_base : fp->_ebuf;
+  _IO_setg(fp, fp->_IO_buf_base, fp->_IO_buf_base, fp->_IO_buf_base);
+  fp->_IO_write_base = fp->_IO_write_ptr = fp->_IO_buf_base;
+  fp->_IO_write_end = (fp->_flags & _IO_LINE_BUF+_IO_UNBUFFERED) ? fp->_IO_buf_base
+    : fp->_IO_buf_end;
   return count != to_do ? EOF : 0;
 }
 
@@ -251,10 +254,10 @@ _IO_file_underflow(fp)
 
   if (fp->_flags & _IO_NO_READS)
     return EOF;
-  if (fp->_gptr < fp->_egptr)
-    return *(unsigned char*)fp->_gptr;
+  if (fp->_IO_read_ptr < fp->_IO_read_end)
+    return *(unsigned char*)fp->_IO_read_ptr;
 
-  if (fp->_base == NULL)
+  if (fp->_IO_buf_base == NULL)
     _IO_doallocbuf(fp);
 
   /* Flush all line buffered files before reading. */
@@ -264,7 +267,8 @@ _IO_file_underflow(fp)
 
   _IO_switch_to_get_mode(fp);
 
-  count = fp->_jumps->__read(fp, fp->_base, fp->_ebuf - fp->_base);
+  count = fp->_jumps->__read(fp, fp->_IO_buf_base,
+			     fp->_IO_buf_end - fp->_IO_buf_base);
   if (count <= 0)
     {
       if (count == 0)
@@ -272,14 +276,15 @@ _IO_file_underflow(fp)
       else
 	fp->_flags |= _IO_ERR_SEEN, count = 0;
   }
-  fp->_eback = fp->_gptr = fp->_base;
-  fp->_egptr = fp->_base + count;
-  fp->_pbase = fp->_pptr = fp->_epptr = fp->_base;
+  fp->_IO_read_base = fp->_IO_read_ptr = fp->_IO_buf_base;
+  fp->_IO_read_end = fp->_IO_buf_base + count;
+  fp->_IO_write_base = fp->_IO_write_ptr = fp->_IO_write_end
+    = fp->_IO_buf_base;
   if (count == 0)
     return EOF;
   if (fp->_offset != _IO_pos_BAD)
     _IO_pos_adjust(fp->_offset, count);
-  return *(unsigned char*)fp->_gptr;
+  return *(unsigned char*)fp->_IO_read_ptr;
 }
 
 int _IO_file_overflow (f, ch)
@@ -288,33 +293,32 @@ int _IO_file_overflow (f, ch)
 {
   if (f->_flags & _IO_NO_WRITES) /* SET ERROR */
     return EOF;
-  /* Allocate a buffer if needed. */
-  if (f->_base == 0)
+  /* If current reading or no buffer allocated. */
+  if ((f->_flags & _IO_CURRENTLY_PUTTING) == 0)
     {
-      _IO_doallocbuf(f);
+      /* Allocate a buffer if needed. */
+      if (f->_IO_buf_base == 0)
+	{
+	  _IO_doallocbuf(f);
+	  f->_IO_read_end = f->_IO_buf_base;
+	  f->_IO_write_ptr = f->_IO_buf_base;
+	}
+      else /* Must be currently reading. */
+	f->_IO_write_ptr = f->_IO_read_ptr;
+      f->_IO_write_base = f->_IO_write_ptr;
+      f->_IO_write_end = f->_IO_buf_end;
+      f->_IO_read_base = f->_IO_read_ptr = f->_IO_read_end;
+
       if (f->_flags & _IO_LINE_BUF+_IO_UNBUFFERED)
-	f->_pbase = f->_pptr = f->_epptr = f->_base;
-      else
-	f->_pbase= f->_pptr = f->_base, f->_epptr = f->_ebuf;
-      f->_eback = f->_gptr = f->_egptr = f->_base;
-      f->_flags |= _IO_CURRENTLY_PUTTING;
-    }
-  /* If currently reading, switch to writing. */
-  else if ((f->_flags & _IO_CURRENTLY_PUTTING) == 0)
-    {
-      if (f->_flags & _IO_LINE_BUF+_IO_UNBUFFERED)
-	f->_pbase = f->_pptr = f->_epptr = f->_gptr;
-      else
-	f->_pbase = f->_pptr = f->_gptr, f->_epptr = f->_ebuf;
-      f->_eback = f->_gptr = f->_egptr;
+	f->_IO_write_end = f->_IO_write_ptr;
       f->_flags |= _IO_CURRENTLY_PUTTING;
     }
   if (ch == EOF)
     return _IO_do_flush(f);
-  if (f->_pptr == f->_ebuf ) /* Buffer is really full */
+  if (f->_IO_write_ptr == f->_IO_buf_end ) /* Buffer is really full */
     if (_IO_do_flush(f) == EOF)
       return EOF;
-  *f->_pptr++ = ch;
+  *f->_IO_write_ptr++ = ch;
   if ((f->_flags & _IO_UNBUFFERED)
       || ((f->_flags & _IO_LINE_BUF) && ch == '\n'))
     if (_IO_do_flush(f) == EOF)
@@ -328,9 +332,9 @@ _IO_file_sync(fp)
 {
   _IO_size_t delta;
   /*    char* ptr = cur_ptr(); */
-  if (fp->_pptr > fp->_pbase)
+  if (fp->_IO_write_ptr > fp->_IO_write_base)
     if (_IO_do_flush(fp)) return EOF;
-  delta = fp->_gptr - fp->_egptr; 
+  delta = fp->_IO_read_ptr - fp->_IO_read_end; 
   if (delta != 0)
     {
 #ifdef TODO
@@ -370,14 +374,14 @@ _IO_file_seekoff(fp, offset, mode)
      end up flushing when we close(), it doesn't make much difference.)
      FIXME: simulate mem-papped files. */
 
-  if (fp->_pptr > fp->_pbase || _IO_in_put_mode(fp))
+  if (fp->_IO_write_ptr > fp->_IO_write_base || _IO_in_put_mode(fp))
     if (_IO_switch_to_get_mode(fp)) return EOF;
 
-  if (fp->_base == NULL)
+  if (fp->_IO_buf_base == NULL)
     {
       _IO_doallocbuf(fp);
-      _IO_setp(fp, fp->_base, fp->_base);
-      _IO_setg(fp, fp->_base, fp->_base, fp->_base);
+      _IO_setp(fp, fp->_IO_buf_base, fp->_IO_buf_base);
+      _IO_setg(fp, fp->_IO_buf_base, fp->_IO_buf_base, fp->_IO_buf_base);
     }
 
   switch (dir)
@@ -419,16 +423,18 @@ _IO_file_seekoff(fp, offset, mode)
 	{
 	  if (_IO_in_backup(fp))
 	    _IO_switch_to_main_get_area(fp);
-	  if (rel_offset <= _egptr - _eback)
+	  if (rel_offset <= _IO_read_end - _IO_read_base)
 	    {
-	      _IO_setg(fp->_base, fp->_base + rel_offset, fp->_egptr);
-	      _IO_setp(fp->_base, fp->_base);
+	      _IO_setg(fp->_IO_buf_base, fp->_IO_buf_base + rel_offset,
+		       fp->_IO_read_end);
+	      _IO_setp(fp->_IO_buf_base, fp->_IO_buf_base);
 	      return offset;
 	    }
 	    /* If we have streammarkers, seek forward by reading ahead. */
 	    if (_IO_have_markers(fp))
 	      {
-		int to_skip = rel_offset - (fp->_gptr - fp->_eback);
+		int to_skip = rel_offset
+		  - (fp->_IO_read_ptr - fp->_IO_read_base);
 		if (ignore(to_skip) != to_skip)
 		  goto dumb;
 		return offset;
@@ -438,7 +444,7 @@ _IO_file_seekoff(fp, offset, mode)
 	{
 	  if (!_IO_in_backup(fp))
 	    _IO_switch_to_backup_area(fp);
-	  gbump(_egptr + rel_offset - gptr());
+	  gbump(fp->_IO_read_end + rel_offset - fp->_IO_read_ptr);
 	  return offset;
 	}
     }
@@ -447,9 +453,9 @@ _IO_file_seekoff(fp, offset, mode)
 #endif
 
   /* Try to seek to a block boundary, to improve kernel page management. */
-  new_offset = offset & ~(fp->_ebuf - fp->_base - 1);
+  new_offset = offset & ~(fp->_IO_buf_end - fp->_IO_buf_base - 1);
   delta = offset - new_offset;
-  if (delta > fp->_ebuf - fp->_base)
+  if (delta > fp->_IO_buf_end - fp->_IO_buf_base)
     {
       new_offset = offset;
       delta = 0;
@@ -461,7 +467,8 @@ _IO_file_seekoff(fp, offset, mode)
     count = 0;
   else
     {
-      count = fp->_jumps->__read(fp, fp->_base, fp->_ebuf - fp->_base);
+      count = fp->_jumps->__read(fp, fp->_IO_buf_base,
+				 fp->_IO_buf_end - fp->_IO_buf_base);
       if (count < delta)
 	{
 	  /* We weren't allowed to read, but try to seek the remainder. */
@@ -470,8 +477,8 @@ _IO_file_seekoff(fp, offset, mode)
 	  goto dumb;
 	}
     }
-  _IO_setg(fp, fp->_base, fp->_base+delta, fp->_base+count);
-  _IO_setp(fp, fp->_base, fp->_base);
+  _IO_setg(fp, fp->_IO_buf_base, fp->_IO_buf_base+delta, fp->_IO_buf_base+count);
+  _IO_setp(fp, fp->_IO_buf_base, fp->_IO_buf_base);
   fp->_offset = result + count;
   _IO_mask_flags(fp, 0, _IO_EOF_SEEN);
   return offset;
@@ -482,8 +489,8 @@ _IO_file_seekoff(fp, offset, mode)
   if (result != EOF)
     _IO_mask_flags(fp, 0, _IO_EOF_SEEN);
   fp->_offset = result;
-  _IO_setg(fp, fp->_base, fp->_base, fp->_base);
-  _IO_setp(fp, fp->_base, fp->_base);
+  _IO_setg(fp, fp->_IO_buf_base, fp->_IO_buf_base, fp->_IO_buf_base);
+  _IO_setp(fp, fp->_IO_buf_base, fp->_IO_buf_base);
   return result;
 }
 
@@ -577,10 +584,10 @@ _IO_file_xsputn(f, data, n)
      (or the filebuf is unbuffered), use sys_write directly. */
 
   /* First figure out how much space is available in the buffer. */
-  count = f->_epptr - f->_pptr; /* Space available. */
+  count = f->_IO_write_end - f->_IO_write_ptr; /* Space available. */
   if ((f->_flags & _IO_LINE_BUF) && (f->_flags & _IO_CURRENTLY_PUTTING))
     {
-      count = f->_ebuf - f->_pptr;
+      count = f->_IO_buf_end - f->_IO_write_ptr;
       if (count >= n)
 	{ register const char *p;
 	  for (p = s + n; p > s; )
@@ -599,16 +606,16 @@ _IO_file_xsputn(f, data, n)
       if (count > to_do)
 	count = to_do;
       if (count > 20) {
-	memcpy(f->_pptr, s, count);
+	memcpy(f->_IO_write_ptr, s, count);
 	s += count;
       }
       else
 	{
-	  register char *p = f->_pptr;
+	  register char *p = f->_IO_write_ptr;
 	  register int i = (int)count;
 	  while (--i >= 0) *p++ = *s++;
 	}
-      f->_pptr += count;
+      f->_IO_write_ptr += count;
       to_do -= count;
     }
   if (to_do + must_flush > 0)
@@ -619,7 +626,7 @@ _IO_file_xsputn(f, data, n)
 
       /* Try to maintain alignment: write a whole number of blocks.
 	 dont_write is what gets left over. */
-      block_size = f->_ebuf - f->_base;
+      block_size = f->_IO_buf_end - f->_IO_buf_base;
       dont_write = block_size >= 128 ? to_do % block_size : 0;
 
       count = to_do - dont_write;
