@@ -25,6 +25,8 @@ the executable file might be covered by the GNU General Public License. */
 /* Generic or default I/O operations. */
 
 #include "libioP.h"
+#include <stdlib.h>
+#include <string.h>
 
 void _IO_un_link(fp)
      _IO_FILE *fp;
@@ -58,7 +60,7 @@ _IO_size_t
 _IO_least_marker(fp)
      register _IO_FILE *fp;
 {
-  _IO_ssize_t least_so_far = fp->_egptr - fp->_eback;
+  _IO_ssize_t least_so_far = fp->_IO_read_end - fp->_IO_read_base;
   register struct _IO_marker *mark;
   for (mark = fp->_markers; mark != NULL; mark = mark->_next)
     if (mark->_pos < least_so_far)
@@ -74,11 +76,11 @@ _IO_switch_to_main_get_area(fp)
 {
   char *tmp;
   fp->_flags &= ~_IO_IN_BACKUP;
-  /* Swap _egptr and _IO_save_end. */
-  tmp = fp->_egptr; fp->_egptr= fp->_IO_save_end; fp->_IO_save_end= tmp;
-  /* Swap _eback and _IO_save_base. */
-  tmp = fp->_eback; fp->_eback = fp->_IO_save_base; fp->_IO_save_base = tmp;
-  fp->_gptr = fp->_eback;
+  /* Swap _IO_read_end and _IO_save_end. */
+  tmp = fp->_IO_read_end; fp->_IO_read_end= fp->_IO_save_end; fp->_IO_save_end= tmp;
+  /* Swap _IO_read_base and _IO_save_base. */
+  tmp = fp->_IO_read_base; fp->_IO_read_base = fp->_IO_save_base; fp->_IO_save_base = tmp;
+  fp->_IO_read_ptr = fp->_IO_read_base;
 }
 
 /* Switch current get area from main get area to (end of) backup area. */
@@ -89,31 +91,31 @@ _IO_switch_to_backup_area(fp)
 {
   char *tmp;
   fp->_flags |= _IO_IN_BACKUP;
-  /* Swap _egptr and _IO_save_end. */
-  tmp = fp->_egptr; fp->_egptr = fp->_IO_save_end; fp->_IO_save_end = tmp;
+  /* Swap _IO_read_end and _IO_save_end. */
+  tmp = fp->_IO_read_end; fp->_IO_read_end = fp->_IO_save_end; fp->_IO_save_end = tmp;
   /* Swap _gbase and _IO_save_base. */
-  tmp = fp->_eback; fp->_eback = fp->_IO_save_base; fp->_IO_save_base = tmp;
-  fp->_gptr = fp->_egptr;
+  tmp = fp->_IO_read_base; fp->_IO_read_base = fp->_IO_save_base; fp->_IO_save_base = tmp;
+  fp->_IO_read_ptr = fp->_IO_read_end;
 }
 
 int
 _IO_switch_to_get_mode(fp)
      register _IO_FILE *fp;
 {
-  if (fp->_pptr > fp->_pbase)
+  if (fp->_IO_write_ptr > fp->_IO_write_base)
     if (fp->_jumps->__overflow(fp, EOF) == EOF)
       return EOF;
   if (_IO_in_backup(fp))
-    fp->_eback = fp->_aux_limit;
+    fp->_IO_read_base = fp->_aux_limit;
   else
     {
-      fp->_eback = fp->_base;
-      if (fp->_pptr > fp->_egptr)
-	fp->_egptr = fp->_pptr;
+      fp->_IO_read_base = fp->_IO_buf_base;
+      if (fp->_IO_write_ptr > fp->_IO_read_end)
+	fp->_IO_read_end = fp->_IO_write_ptr;
     }
-  fp->_gptr = fp->_pptr;
+  fp->_IO_read_ptr = fp->_IO_write_ptr;
 
-  fp->_pbase = fp->_pptr = fp->_epptr = fp->_gptr;
+  fp->_IO_write_base = fp->_IO_write_ptr = fp->_IO_write_end = fp->_IO_read_ptr;
 
   fp->_flags &= ~_IO_CURRENTLY_PUTTING;
   return 0;
@@ -136,13 +138,13 @@ int
 _IO_switch_to_put_mode(fp)
      register _IO_FILE *fp;
 {
-  fp->_pbase = fp->_gptr;
-  fp->_pptr = fp->_gptr;
+  fp->_IO_write_base = fp->_IO_read_ptr;
+  fp->_IO_write_ptr = fp->_IO_read_ptr;
   /* Following is wrong if line- or un-buffered? */
-  fp->_epptr = fp->_flags & _IO_IN_BACKUP ? fp->_egptr : fp->_ebuf;
+  fp->_IO_write_end = fp->_flags & _IO_IN_BACKUP ? fp->_IO_read_end : fp->_IO_buf_end;
 
-  fp->_gptr = fp->_egptr;
-  fp->_eback = fp->_egptr;
+  fp->_IO_read_ptr = fp->_IO_read_end;
+  fp->_IO_read_base = fp->_IO_read_end;
 
   fp->_flags |= _IO_CURRENTLY_PUTTING;
   return 0;
@@ -162,20 +164,20 @@ __underflow(fp)
 {
   if (_IO_in_put_mode(fp))
     if (_IO_switch_to_get_mode(fp) == EOF) return EOF;
-  if (fp->_gptr < fp->_egptr)
-    return *(unsigned char*)fp->_gptr;
+  if (fp->_IO_read_ptr < fp->_IO_read_end)
+    return *(unsigned char*)fp->_IO_read_ptr;
   if (_IO_in_backup(fp))
     {
       _IO_switch_to_main_get_area(fp);
-      if (fp->_gptr < fp->_egptr)
-	return *fp->_gptr;
+      if (fp->_IO_read_ptr < fp->_IO_read_end)
+	return *fp->_IO_read_ptr;
     }
   if (_IO_have_markers(fp))
     {
-      /* Append [_gbase.._egptr] to backup area. */
+      /* Append [_gbase.._IO_read_end] to backup area. */
       int least_mark = _IO_least_marker(fp);
       /* needed_size is how much space we need in the backup area. */
-      int needed_size = (fp->_egptr - fp->_eback) - least_mark;
+      int needed_size = (fp->_IO_read_end - fp->_IO_read_base) - least_mark;
       int current_Bsize = fp->_IO_save_end - fp->_IO_save_base;
       int avail; /* Extra space available for future expansion. */
       int delta;
@@ -193,12 +195,12 @@ __underflow(fp)
 		     fp->_IO_save_end + least_mark,
 		     -least_mark);
 	      memcpy(new_buffer +avail - least_mark,
-		     fp->_eback,
-		     fp->_egptr - fp->_eback);
+		     fp->_IO_read_base,
+		     fp->_IO_read_end - fp->_IO_read_base);
 	    }
 	  else
 	    memcpy(new_buffer + avail,
-		   fp->_eback + least_mark,
+		   fp->_IO_read_base + least_mark,
 		   needed_size);
 	  free (fp->_IO_save_base);
 	  fp->_IO_save_base = new_buffer;
@@ -213,18 +215,18 @@ __underflow(fp)
 		      fp->_IO_save_end + least_mark,
 		      -least_mark);
 	      memcpy(fp->_IO_save_base + avail - least_mark,
-		     fp->_eback,
-		     fp->_egptr - fp->_eback);
+		     fp->_IO_read_base,
+		     fp->_IO_read_end - fp->_IO_read_base);
 	    }
 	  else if (needed_size > 0)
 	    memcpy(fp->_IO_save_base + avail,
-		   fp->_eback + least_mark,
+		   fp->_IO_read_base + least_mark,
 		   needed_size);
 	}
       /* FIXME: Dubious arithmetic if pointers are NULL */
       fp->_aux_limit = fp->_IO_save_base + avail;
       /* Adjust all the streammarkers. */
-      delta = fp->_egptr - fp->_eback;
+      delta = fp->_IO_read_end - fp->_IO_read_base;
       for (mark = fp->_markers; mark != NULL; mark = mark->_next)
 	mark->_pos -= delta;
     }
@@ -240,10 +242,10 @@ _IO_setb(f, b, eb, a)
      char *eb;
      int a;
 {
-  if (f->_base && !(f->_flags & _IO_USER_BUF))
-    FREE_BUF(f->_base);
-  f->_base = b;
-  f->_ebuf = eb;
+  if (f->_IO_buf_base && !(f->_flags & _IO_USER_BUF))
+    FREE_BUF(f->_IO_buf_base);
+  f->_IO_buf_base = b;
+  f->_IO_buf_end = eb;
   if (a)
     f->_flags &= ~_IO_USER_BUF;
   else
@@ -254,7 +256,7 @@ void
 _IO_doallocbuf(fp)
      register _IO_FILE *fp;
 {
-  if (fp->_base)
+  if (fp->_IO_buf_base)
     return;
   if (!(fp->_flags & _IO_UNBUFFERED))
     if (fp->_jumps->__doallocate(fp) != EOF)
@@ -274,25 +276,25 @@ _IO_default_xsputn(f, data, n)
     return 0;
   for (;;)
     {
-      _IO_size_t count = f->_epptr - f->_pptr; /* Space available. */
+      _IO_size_t count = f->_IO_write_end - f->_IO_write_ptr; /* Space available. */
       if (count > 0)
 	{
 	  if (count > more)
 	    count = more;
 	  if (count > 20)
 	    {
-	      memcpy(f->_pptr, s, count);
+	      memcpy(f->_IO_write_ptr, s, count);
 	      s += count;
-	      f->_pptr += count;
+	      f->_IO_write_ptr += count;
             }
 	  else if (count <= 0)
 	    count = 0;
 	  else
 	    {
-	      register char *p = f->_pptr;
+	      register char *p = f->_IO_write_ptr;
 	      register _IO_ssize_t i;
 	      for (i = count; --i >= 0; ) *p++ = *s++;
-	      f->_pptr = p;
+	      f->_IO_write_ptr = p;
             }
 	  more -= count;
         }
@@ -323,25 +325,25 @@ _IO_default_xsgetn(fp, data, n)
   register char *s = data;
   for (;;)
     {
-      _IO_size_t count = fp->_egptr - fp->_gptr; /* Data available. */
+      _IO_size_t count = fp->_IO_read_end - fp->_IO_read_ptr; /* Data available. */
       if (count > 0)
 	{
 	  if (count > more)
 	    count = more;
 	  if (count > 20)
 	    {
-	      memcpy(s, fp->_gptr, count);
+	      memcpy(s, fp->_IO_read_ptr, count);
 	      s += count;
-	      fp->_gptr += count;
+	      fp->_IO_read_ptr += count;
 	    }
 	  else if (count <= 0)
 	    count = 0;
 	  else
 	    {
-	      register char *p = fp->_gptr;
+	      register char *p = fp->_IO_read_ptr;
 	      register int i = (int)count;
 	      while (--i >= 0) *s++ = *p++;
-	      fp->_gptr = p;
+	      fp->_IO_read_ptr = p;
             }
             more -= count;
         }
@@ -355,7 +357,7 @@ int
 _IO_sync(fp)
      register _IO_FILE *fp;
 {
-  if (fp->_gptr == fp->_egptr && fp->_pptr == fp->_pbase)
+  if (fp->_IO_read_ptr == fp->_IO_read_end && fp->_IO_write_ptr == fp->_IO_write_base)
     return 0;
   return EOF;
 }
@@ -378,8 +380,8 @@ _IO_default_setbuf(fp, p, len)
 	fp->_flags &= ~_IO_UNBUFFERED;
 	_IO_setb(fp, p, p+len, 0);
       }
-    fp->_pbase = fp->_pptr = fp->_epptr = 0;
-    fp->_eback = fp->_gptr = fp->_egptr = 0;
+    fp->_IO_write_base = fp->_IO_write_ptr = fp->_IO_write_end = 0;
+    fp->_IO_read_base = fp->_IO_read_ptr = fp->_IO_read_end = 0;
     return 0;
 }
 
@@ -409,14 +411,14 @@ _IO_init(fp, flags)
      int flags;
 {
   fp->_flags = _IO_MAGIC|flags;
-  fp->_base = NULL;
-  fp->_ebuf = NULL;
-  fp->_eback = NULL;
-  fp->_gptr = NULL;
-  fp->_egptr = NULL;
-  fp->_pbase = NULL;
-  fp->_pptr = NULL;
-  fp->_epptr = NULL;
+  fp->_IO_buf_base = NULL;
+  fp->_IO_buf_end = NULL;
+  fp->_IO_read_base = NULL;
+  fp->_IO_read_ptr = NULL;
+  fp->_IO_read_end = NULL;
+  fp->_IO_write_base = NULL;
+  fp->_IO_write_ptr = NULL;
+  fp->_IO_write_end = NULL;
   fp->_chain = NULL; /* Not necessary. */
 
   fp->_IO_save_base = NULL;
@@ -426,12 +428,19 @@ _IO_init(fp, flags)
   fp->_cur_column = 0;
 }
 
+int
+_IO_default_sync (fp)
+     _IO_FILE *fp;
+{
+  return 0;
+}
+
 void _IO_default_finish (fp)
      _IO_FILE *fp;
 {
   struct _IO_marker *mark;
-  if (fp->_base && !(fp->_flags & _IO_USER_BUF))
-    FREE_BUF(fp->_base);
+  if (fp->_IO_buf_base && !(fp->_flags & _IO_USER_BUF))
+    FREE_BUF(fp->_IO_buf_base);
 
   for (mark = fp->_markers; mark != NULL; mark = mark->_next)
     mark->_sbuf = NULL;
@@ -455,10 +464,10 @@ _IO_sputbackc(fp, c) /* ungetc(c, fp) */
      register _IO_FILE *fp;
      int c;
 {
-  if (fp->_gptr > fp->_eback
-      && (unsigned char)fp->_gptr[-1] == (unsigned char)c)
+  if (fp->_IO_read_ptr > fp->_IO_read_base
+      && (unsigned char)fp->_IO_read_ptr[-1] == (unsigned char)c)
     {
-      fp->_gptr--;
+      fp->_IO_read_ptr--;
       return (unsigned char)c;
     }
   return fp->_jumps->__pbackfail(fp, c);
@@ -468,10 +477,10 @@ int
 _IO_sungetc(fp)
      register _IO_FILE *fp;
 {
-  if (fp->_gptr > fp->_eback)
+  if (fp->_IO_read_ptr > fp->_IO_read_base)
     {
-      fp->_gptr--;
-      return (unsigned char)*fp->_gptr;
+      fp->_IO_read_ptr--;
+      return (unsigned char)*fp->_IO_read_ptr;
     }
   else
     return fp->_jumps->__pbackfail(fp, EOF);
@@ -486,7 +495,7 @@ _IO_set_column(fp, c)
   if (c == -1)
     fp->_collumn = -1;
   else
-    fp->_collumn = c - (fp->_pptr - fp->_pbase);
+    fp->_collumn = c - (fp->_IO_write_ptr - fp->_IO_write_base);
 }
 #else
 int
@@ -519,8 +528,8 @@ _IO_get_column(fp)
 {
   if (fp->_cur_column) 
     return _IO_adjust_column(fp->_cur_column - 1,
-			      fp->_pbase,
-			      fp->_pptr - fp->_pbase);
+			      fp->_IO_write_base,
+			      fp->_IO_write_ptr - fp->_IO_write_base);
   return -1;
 }
 
@@ -619,14 +628,15 @@ int _IO_seekmark(fp, mark, delta)
     {
       if (_IO_in_backup(fp))
 	_IO_switch_to_main_get_area(fp);
-      fp->_gptr = fp->_eback + mark->_pos;
+      fp->_IO_read_ptr = fp->_IO_read_base + mark->_pos;
     }
   else
     {
       if (!_IO_in_backup(fp))
 	_IO_switch_to_backup_area(fp);
-      fp->_gptr = fp->_egptr + mark->_pos;
+      fp->_IO_read_ptr = fp->_IO_read_end + mark->_pos;
     }
+  return 0;
 }
 
 void _IO_unsave_markers(fp)
@@ -660,10 +670,10 @@ _IO_nobackup_pbackfail(fp, c)
      register _IO_FILE *fp;
      int c;
 {
-  if (fp->_gptr > fp->_eback)
-	fp->_gptr--;
-  if (c != EOF && *fp->_gptr != c)
-      *fp->_gptr = c;
+  if (fp->_IO_read_ptr > fp->_IO_read_base)
+	fp->_IO_read_ptr--;
+  if (c != EOF && *fp->_IO_read_ptr != c)
+      *fp->_IO_read_ptr = c;
   return (unsigned char)c;
 }
 
@@ -672,7 +682,7 @@ _IO_default_pbackfail(fp, c)
      register _IO_FILE *fp;
      int c;
 {
-  if (fp->_gptr <= fp->_eback)
+  if (fp->_IO_read_ptr <= fp->_IO_read_base)
       {
 	/* Need to handle a filebuf in write mode (switch to read mode). FIXME!*/
 	if (_IO_have_backup(fp) && !_IO_in_backup(fp))
@@ -691,27 +701,27 @@ _IO_default_pbackfail(fp, c)
 	    fp->_aux_limit = fp->_IO_save_end;
 	    _IO_switch_to_backup_area(fp);
 	  }
-	else if (fp->_gptr <= fp->_eback)
+	else if (fp->_IO_read_ptr <= fp->_IO_read_base)
 	  {
 	    /* Increase size of existing backup buffer. */
 	    _IO_size_t new_size;
-	    _IO_size_t old_size = fp->_egptr - fp->_eback;
+	    _IO_size_t old_size = fp->_IO_read_end - fp->_IO_read_base;
 	    char *new_buf;
 	    new_size = 2 * old_size;
 	    new_buf = (char*)malloc(new_size);
 	    if (new_buf == NULL)
 	      return EOF;
-	    memcpy(new_buf+(new_size-old_size), fp->_eback, old_size);
-	    free (fp->_eback);
+	    memcpy(new_buf+(new_size-old_size), fp->_IO_read_base, old_size);
+	    free (fp->_IO_read_base);
 	    _IO_setg(fp,
 		     new_buf, new_buf+(new_size-old_size), new_buf+new_size);
-	    fp->_aux_limit = fp->_gptr;
+	    fp->_aux_limit = fp->_IO_read_ptr;
 	  }
       }
-  fp->_gptr--;
-  if (c != EOF && *fp->_gptr != c)
-    *fp->_gptr = c;
-  return (unsigned char)*fp->_gptr;
+  fp->_IO_read_ptr--;
+  if (c != EOF && *fp->_IO_read_ptr != c)
+    *fp->_IO_read_ptr = c;
+  return (unsigned char)*fp->_IO_read_ptr;
 }
 
 _IO_pos_t
