@@ -34,6 +34,6 @@ _IO_fread(buf, size, count, fp)
   _IO_size_t bytes_requested = size*count;
   _IO_size_t bytes_read;
   COERCE_FILE(fp);
-  bytes_read = fp->_jumps->__xsgetn(fp, (char *)buf, bytes_requested);
+  bytes_read = _IO_sgetn(fp, (char *)buf, bytes_requested);
   return bytes_requested == bytes_read ? count : bytes_read / size;
 }
