@@ -37,7 +37,7 @@ _IO_getline(fp, buf, n, delim, extract_delim)
      _IO_FILE *fp;
      char* buf;
      _IO_size_t n;
-     char delim;
+     int delim;
      int extract_delim;
 {
   register char *ptr = buf;
