@@ -40,7 +40,7 @@ the executable file might be covered by the GNU General Public License. */
  */
 
 /* Extensively hacked for GNU iostream by Per Bothner 1991, 1992, 1993.
-/* Changes copyright Per Bothner 1992, 1993. */
+   Changes copyright Free Software Foundation 1992, 1993. */
 
 #if defined(LIBC_SCCS) && !defined(lint)
 static char sccsid[] = "%W% (Berkeley) %G%";
@@ -102,7 +102,7 @@ static char sccsid[] = "%W% (Berkeley) %G%";
 #define u_long unsigned long
 
 extern u_long strtoul _PARAMS((const char*, char**, int));
-extern u_long strtol _PARAMS((const char*, char**, int));
+extern long strtol _PARAMS((const char*, char**, int));
 static const u_char *__sccl _PARAMS((char *tab, const u_char *fmt));
 #ifndef USE_DTOA
 extern double atof();
