@@ -35,5 +35,5 @@ _IO_vsscanf (string, format, args)
   _IO_init((_IO_FILE*)&sf, 0);
   ((_IO_FILE*)&sf)->_jumps = &_IO_str_jumps;
   _IO_str_init_static ((_IO_FILE*)&sf, (char*)string, -1, NULL);
-  return _IO_vfscanf((_IO_FILE*)&sf, format, args);
+  return _IO_vfscanf((_IO_FILE*)&sf, format, args, NULL);
 }
