@@ -27,18 +27,18 @@ the executable file might be covered by the GNU General Public License. */
 #include "streambuf.h"
 #include <stdarg.h>
 
-int streambuf::vscan(char const *fmt0, _IO_va_list ap, ios* stream = NULL)
+int streambuf::vscan(char const *fmt0, _IO_va_list ap, ios* stream /* = NULL*/)
 {
-  return _IO_vfscanf(this, fmt0, ap);
-#ifdef TODO
-  stream = ...;
-#endif
+  int errcode = 0;
+  return _IO_vfscanf(this, fmt0, ap, &errcode);
+  if (stream)
+    stream->setstate((ios::iostate)errcode);
 }
 int streambuf::scan(char const *format ...)
 {
   va_list ap;
   va_start(ap, format);
-  int count = _IO_vfscanf(this, format, ap);
+  int count = _IO_vfscanf(this, format, ap, NULL);
   va_end(ap);
   return count;
 }
