@@ -23,6 +23,7 @@ This exception does not however invalidate any other reasons why
 the executable file might be covered by the GNU General Public License. */
 
 #include "libioP.h"
+#include "stdio.h"
 
 int
 vfscanf(fp, format, args)
@@ -31,5 +32,5 @@ vfscanf(fp, format, args)
      _IO_va_list args;
 {
   COERCE_FILE(fp);
-  return _IO_vfscanf(fp, format, args);
+  return _IO_vfscanf(fp, format, args, NULL);
 }
