@@ -194,7 +194,8 @@ class ios : public _ios_fields {
 #else
     void _throw_failure() const { }
 #endif
-    void _IO_fix_vtable();
+    void _IO_fix_vtable();  /* TEMPORARY - for binary compatibility */
+    void _IO_fix_vtable() const;
     streambuf* rdbuf() const;
 #if 0
     streambuf* rdbuf(streambuf *_s) {
@@ -286,7 +287,6 @@ struct streambuf : public _IO_FILE { // protected??
     _IO_FILE*& xchain() { return _chain; }
     void _un_link();
     void _link_in();
-#if 1
     char* gptr() const
       { return _IO_file_flags & _IO_IN_BACKUP ? _IO_save_base : _IO_read_ptr; }
     char* pptr() const { return _IO_write_ptr; }
@@ -314,39 +314,18 @@ struct streambuf : public _IO_FILE { // protected??
     void setg(char* eb, char* g, char *eg) {
       if (_IO_file_flags & _IO_IN_BACKUP) _IO_free_backup_area(this); 
       _IO_read_base = eb; _IO_read_ptr = g; _IO_read_end = eg; }
-#else
-    char* gptr() const { return _gptr; }
-    char* pptr() const { return _pptr; }
-    char* egptr() const { return _egptr; }
-    char* epptr() const { return _epptr; }
-    char* pbase() const { return _pbase; }
-    char* eback() const { return _eback; }
-    char* base() const { return _base; }
-    char* ebuf() const { return _ebuf; }
-    int blen() const { return _ebuf - _base; }
-    void xput_char(char c) { *_pptr++ = c; }
-    int xflags() { return _flags; }
-    int xflags(int f) { int fl = _flags; _flags = f; return fl; }
-    void xsetflags(int f) { _flags |= f; }
-    void xsetflags(int f, int mask) { _flags = (_flags & ~mask) | (f & mask); }
-    void gbump(int n) { _gptr += n; }
-    void pbump(int n) { _pptr += n; }
-    void setb(char* b, char* eb, int a=0);
-    void setp(char* p, char* ep) { _pbase=_pptr=p; _epptr=ep; }
-    void setg(char* eb, char* g, char *eg) { _eback=eb; _gptr=g; _egptr=eg; }
-#endif
     char *shortbuf() { return _shortbuf; }
 
     int in_backup() { return _flags & _IO_IN_BACKUP; }
     // The start of the main get area:  FIXME:  wrong for write-mode filebuf?
-    char *Gbase() { return in_backup() ? _other_gbase : _eback; }
+    char *Gbase() { return in_backup() ? _other_gbase : _IO_read_base; }
     // The end of the main get area:
-    char *eGptr() { return in_backup() ? _other_egptr : _egptr; }
+    char *eGptr() { return in_backup() ? _other_egptr : _IO_read_end; }
     // The start of the backup area:
-    char *Bbase() { return in_backup() ? _eback : _other_gbase; }
+    char *Bbase() { return in_backup() ? _IO_read_base : _other_gbase; }
     char *Bptr() { return _aux_limit; }
     // The end of the backup area:
-    char *eBptr() { return in_backup() ? _egptr : _other_egptr; }
+    char *eBptr() { return in_backup() ? _IO_read_end : _other_egptr; }
     char *Nbase() { return _other_gbase; }
     char *eNptr() { return _other_egptr; }
     int have_backup() { return _other_gbase != NULL; }
@@ -387,8 +366,8 @@ struct streambuf : public _IO_FILE { // protected??
     virtual int sync();
     virtual int pbackfail(int c);
     virtual streambuf* setbuf(char* p, int len);
-    int in_avail() { return _egptr - _gptr; }
-    int out_waiting() { return _pptr - _pbase; }
+    int in_avail() { return _IO_read_end - _IO_read_ptr; }
+    int out_waiting() { return _IO_write_ptr - _IO_write_base; }
     virtual streamsize xsputn(const char* s, streamsize n);
     streamsize sputn(const char* s, streamsize n) { return xsputn(s, n); }
     streamsize padn(char pad, streamsize n) { return _IO_padn(this, pad, n); }
@@ -402,9 +381,10 @@ struct streambuf : public _IO_FILE { // protected??
     int sbumpc() { return _IO_getc(this); }
     int sgetc() { return _IO_peekc(this); }
     int snextc() {
-	if (_gptr >= _egptr && __underflow(this) == EOF) return EOF;
-	else return _gptr++, sgetc(); }
-    void stossc() { if (_gptr < _egptr) _gptr++; }
+	if (_IO_read_ptr >= _IO_read_end && __underflow(this) == EOF)
+	  return EOF;
+	else return _IO_read_ptr++, sgetc(); }
+    void stossc() { if (_IO_read_ptr < _IO_read_end) _IO_read_ptr++; }
     int vscan(char const *fmt0, _IO_va_list ap, ios* stream = NULL);
     int scan(char const *fmt0 ...);
     int vform(char const *fmt0, _IO_va_list ap);
@@ -423,22 +403,6 @@ struct streambuf : public _IO_FILE { // protected??
 // A backupbuf is a streambuf with full backup and savepoints on reading.
 // All standard streambufs in the GNU iostream library are backupbufs.
 
-#if 0
-// A backupbuf may have two get area:
-// - The main get area, and (sometimes) the putback area.
-// Whichever one of these contains the gptr is the current get area;
-// the other one is the non-current get area.
-
-class backupbuf : public streambuf {
-    friend class streammarker;
-  protected:
-    backupbuf(int flags=0) : streambuf(flags|_IO_IS_BACKUPBUF) { }
-  public:
-    virtual int underflow();
-    virtual int overflow(int c = EOF);
-};
-#endif
-
 class filebuf : public streambuf {
   protected:
     void init();
@@ -470,7 +434,6 @@ class filebuf : public streambuf {
     /* System's idea of pointer */
     char* file_ptr() { return eGptr(); }
     int do_write(const char *data, int to_do);
-//    int do_flush() { return do_write(_pbase, _pptr-_pbase); }
     // Low-level operations (Usually invoke system calls.)
     virtual streamsize sys_read(char* buf, streamsize size);
     virtual streampos sys_seek(streamoff, _seek_dir);
@@ -487,7 +450,7 @@ inline ios::ios(streambuf* sb /* = 0 */, ostream* tie_to /* = 0 */) {
 inline streambuf* ios::rdbuf() const
 {
   if (_strbuf && _strbuf->_vtable() == 0)
-    ((ios*)this)->_IO_fix_vtable();
+    _IO_fix_vtable();
   return _strbuf;
 }
 
